pro demmap_example, ncols=ncols,nrows=nrows,nbridges=nbridges

  ; Demo code that makes a synthetic AIA map using a Gaussian model DEM
  ; Then perform the regularization and compares the results
  ; 25-May-2012 IGH
  ; 03-Jul-2012 IGH - Updated for return of regularized maps

  if n_elements(nbridges) eq 0 then nbridges=2
  if n_elements(ncols) eq 0 then ncols=10
  if n_elements(nrows) eq 0 then nrows=100
  
  
  ;  ############### Make the DEM model ############################
  d1=6d22
  m1=6.5
  s1=0.15
  
  temps=[0.5,1,1.5,2,3,4,6,8,11,14,19,25,32]*1e6
  
  logt0=alog10(temps)
  
  resp_file=file_search('aia_resp_chi.dat')
  if strlen(resp_file) eq 0 then begin
    eff=aia_get_response(/temperature,/dn,/chiantifix,/evenorm)
    save,file='aia_resp_chi.dat',eff
  endif else  begin
    restore,file='aia_resp_chi.dat'
  endelse
  
  idc=[0,1,2,3,4,6]
  logT=eff.logte
  gdt=where (logt ge min(logt0) and logt le max(logt0),ngd)
  logt=logt[gdt]
  TRmatrix=eff.all[*,idc]
  TRmatrix=TRmatrix[gdt,*]
  root2pi=sqrt(2.*!PI)
  dem_mod=(d1/(root2pi*s1))*exp(-(logT-m1)^2/(2*s1^2))
  ;  ############### Make the synthetic data from the DEM model #############
  dn_mod=dem2dn(logT, dem_mod, TRmatrix)
  dn=dn_mod.dn
  ; ################ Mathe the map from the synthetic data ##############
  map=replicate(make_map(fltarr(ncols,nrows),dx=0.6,dy=0.6,yc=0.,xc=0.),6)
  map.dur=1.
  for i=0,ncols-1 do for j=0,nrows-1 do map.data[i,j]=dn
  
  
  ; ######## do the regularization ###########################
  dn2dem_map,map,dem,chisq,temps=temps,edem=edem,elogt=elogt,$
    nbridges=nbridges,mapreg=mapreg
    
    
  linecolors
  midt=get_edges(logt0,/mean)
  nt=n_elements(midt)
  !p.thick=2
  !p.charsize=1.5
  yr=max(dem_mod)*[1e-3,1.5]
  
  ; just plot the DEM in the first pixel
  ; all other pixels will be identical in this test
  plot,logt,dem_mod,yrange=yr,xrange=[5.5,7.6],xstyle=17,/ylog,$
    ytitle='DEM [cm!U-5!N K!U-1!N]', xtitle='log!D10!N T [K]'
  oplot,midt,dem[0,0,*],color=2,psym=-4
  mined=reform((dem[0,0,*]-edem[0,0,*]) > yr[0])
  maxed=reform((dem[0,0,*]+edem[0,0,*]) < yr[1])
  for i=0, nt-1 do oplot,midt[i]*[1,1],[mined[i],maxed[i]],color=2
  elmin=   (midt-elogt[0,0,*])
  elmax=(midt+elogt[0,0,*])
  for i=0, nt-1 do $
    oplot,[elmin[i] < logt0[i] ,elmax[i]>logt0[i+1]],dem[0,0,i]*[1,1],color=2
    
  ; The input data (same in all pixels)
  ; print, map[*].data[0,0]
  ; What the regularized dem solution would look like to AIA
  ; print,mapreg[*].data[0,0]
    
    
  stop
end

